/*-----------------------------------------------------------------------
 *
 * File Name: EllipsoidOverlapTools.h
 *
 * Author: Robinson, C. A., and Sengupta, A.
 *
 * Revision: $Id: EllipsoidOverlapTools.h,v 1.6 2006/09/01 12:19:03 spxcar Exp $
 *
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="EllipsoidOverlapToolsHV">
Author: Robinson, C. A. and Sengupta, A. S.
$Id: EllipsoidOverlapTools.h,v 1.6 2006/09/01 12:19:03 spxcar Exp $
</lalVerbatim>
#endif

#ifndef _ELLIPSOIDOVERLAPTOOLS_H
#define _ELLIPSOIDOVERLAPTOOLS_H


#include    <math.h>
#include    <lal/LALStdlib.h>
#include    <lal/LALGSL.h>
#include    <lal/LALError.h>

#include    <gsl/gsl_errno.h>
#include    <gsl/gsl_math.h>
#include    <gsl/gsl_min.h>
#include    <gsl/gsl_vector.h>
#include    <gsl/gsl_matrix.h>
#include    <gsl/gsl_blas.h>
#include    <gsl/gsl_linalg.h>

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID( ELLIPSOIDOVERLAPTOOLSH, "$Id: EllipsoidOverlapTools.h,v 1.6 2006/09/01 12:19:03 spxcar Exp $" );

typedef struct tagfContactWorkSpace
{
    /* Dimension of the matrices & vectors */
    INT4              n;

    /* Vector r_AB = r_B - r_A and           */
    /* shape matrices for ellipsoid centered */
    /* at points A and B                     */
    gsl_vector        *r_AB;
    const gsl_matrix  *invQ1, *invQ2;

    /* Parameters for minimizing the contact function */
    REAL8             convParam;
    const             gsl_min_fminimizer_type *T;
    gsl_min_fminimizer  *s;

    /* Temporary workspace variables */
    gsl_matrix        *tmpA, *tmpB, *C;
    gsl_vector        *tmpV;
    gsl_permutation   *p1;
}
fContactWorkSpace;


/* Function Prototypes */
REAL8 XLALCheckOverlapOfEllipsoids (
        const gsl_vector   *ra, 
        const gsl_vector   *rb,
        fContactWorkSpace  *workSpace );


fContactWorkSpace * XLALInitFContactWorkSpace(
                       INT4                           n,
                       const gsl_matrix              *a,
                       const gsl_matrix              *b,
                       const gsl_min_fminimizer_type *T,
                       REAL8                          conv
                                             );

void XLALFreeFContactWorkSpace( fContactWorkSpace *workSpace );

#ifdef  __cplusplus
}
#endif

#endif   /* _ELLIPSOIDOVERLAPTOOLS_H */
